/** ###################################################################
**     Filename  : Keypad.C
**     Project   : DC17
**     Processor : MC56F8006_32_LQFP
**     Version   : Driver 01.13
**     Compiler  : Metrowerks DSP C Compiler
**     Date/Time : 1/18/2009, 1:37 PM
**     Abstract  :
**         Main module.
**         This module contains user's application code.
**     Settings  :
**     Contents  :
**         No public methods
**
**     (c) Copyright UNIS, a.s. 1997-2008
**     UNIS, a.s.
**     Jundrovska 33
**     624 00 Brno
**     Czech Republic
**     http      : www.processorexpert.com
**     mail      : info@processorexpert.com
** ###################################################################*/
/* MODULE Keypad */

/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "MICOUT.h"
#include "MEM1.h"
#include "Term1.h"
#include "Inhr6.h"
#include "TI1.h"
#include "PORTC6IN.h"
#include "PORTCOUT.h"
#include "PORTA4OUT.h"
#include "KeypadColumnsIn.h"
#include "KeypadRowOut0.h"
#include "KeypadRowOut1.h"
#include "KeypadRowOut2.h"
#include "KeypadRowOut3.h"
#include "MICPWR.h"
#include "DFR1.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

#include "Keypad.h"

/* local defines */
#define MAX_SCAN_VALUE_DEBOUNCE_COUNT		4

static UWord16 PreviousScanValue = 0;
static UWord8 ScanValueDebounceCount = 0;


void InitializeKeypad(void)
{
	PreviousScanValue = 0;
	
	/* disable row output drive by configuring as input */
	KeypadRowOut0_SetDir(FALSE);
	KeypadRowOut1_SetDir(FALSE);
	KeypadRowOut2_SetDir(FALSE);
	KeypadRowOut3_SetDir(FALSE);
	
	/* set row output register values to 0 */
	KeypadRowOut0_ClrVal();
	KeypadRowOut1_ClrVal();
	KeypadRowOut2_ClrVal();
	KeypadRowOut3_ClrVal();
}


UWord16 ScanKeypad(void)
{
	UWord16 ReturnValue = 0;
	UWord16 ScanValue = 0;
	UWord8 ColumnResult;
	UWord8 i;
	
	/* perform physical keypad scan */
	for(i = 0;i < KEYPAD_ROW_COUNT;i++)
	{
		/* activate row output corresponding to our index */
		switch(i)
		{
			default:
			case 0:
				KeypadRowOut1_SetDir(FALSE);
				KeypadRowOut2_SetDir(FALSE);
				KeypadRowOut3_SetDir(FALSE);
				
				KeypadRowOut0_SetDir(TRUE);
				
				break;
			
			case 1:
				KeypadRowOut0_SetDir(FALSE);
				KeypadRowOut2_SetDir(FALSE);
				KeypadRowOut3_SetDir(FALSE);

				KeypadRowOut1_SetDir(TRUE);
				break;

			case 2:
				KeypadRowOut0_SetDir(FALSE);
				KeypadRowOut1_SetDir(FALSE);
				KeypadRowOut3_SetDir(FALSE);
				
				KeypadRowOut2_SetDir(TRUE);
				
				break;

			case 3:
				KeypadRowOut0_SetDir(FALSE);
				KeypadRowOut1_SetDir(FALSE);
				KeypadRowOut2_SetDir(FALSE);

				KeypadRowOut3_SetDir(TRUE);
				break;
		}
		
		ColumnResult = KeypadColumnsIn_GetVal();
		ColumnResult = (UWord8)~ColumnResult;
		ColumnResult &= KEYPAD_COLUMN_READ_RESULTS_MASK;
		
		ScanValue |= ((ColumnResult & KEYPAD_COLUMN_READ_RESULTS_MASK) << (i * KEYPAD_COLUMN_COUNT));
	}
	
	if(ScanValue == PreviousScanValue)
	{
		ScanValueDebounceCount++;
	}
	else
	{
		ScanValueDebounceCount = 0;
	}

	if(ScanValueDebounceCount > MAX_SCAN_VALUE_DEBOUNCE_COUNT)
	{
		ScanValueDebounceCount = MAX_SCAN_VALUE_DEBOUNCE_COUNT;
		ReturnValue = ScanValue;
	}

	PreviousScanValue = ScanValue;

	return ReturnValue;
}


/*
** ###################################################################
**
**     This file was created by UNIS Processor Expert 2.99 [04.17]
**     for the Freescale 56800 series of microcontrollers.
**
** ###################################################################
*/
