/** ###################################################################
**     Filename  : Redbox.c
**     Project   : DDSTest
**     Processor : MC56F8006_48_LQFP
**     Version   : Driver 01.13
**     Compiler  : Metrowerks DSP C Compiler
**     Date/Time : 7/24/2009, 4:32 PM
**     Abstract  :
**         Main module.
**         This module contains user's application code.
**     Settings  :
**     Contents  :
**         No public methods
**
**     (c) Copyright UNIS, a.s. 1997-2008
**     UNIS, a.s.
**     Jundrovska 33
**     624 00 Brno
**     Czech Republic
**     http      : www.processorexpert.com
**     mail      : info@processorexpert.com
** ###################################################################*/
/* MODULE Redbox */


/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

#include "Redbox.h"
#include "ToneGenerate.h"

// Redbox on tones = 1700 Hz & 2200 Hz (handled by Playtone() in KEYTONE_MODE_REDBOX)
// On-off sequences:
// Nickel ('5' key) = 66ms on / 66 ms off
// Dime = ('1' key) = 2 * (66ms on / 66 ms off)
// Quarter = ('2' key) = 5 * (33 ms on / 33 ms off)

/* static variables */
static RedboxFSMState_t RedboxState = REDBOX_WAITING_FOR_VALID_KEYPRESS;
static UWord16 TargetOnOffIntervalTickCount = 0;
static UWord16 OnOffIntervalTickCount = 0;
static UWord8 IntervalRepeatCount = 0;

#ifdef _SIMPLE_REDBOX_TEST
void RunRedboxFSM(Keypress_t Keycode, UWord16 SampleRate)
{
	// simple handler for testing generation of redbox tones
	// play tones if '0' key pressed
	PlayTone(Keycode, KEYTONE_MODE_REDBOX, SampleRate);
}

#else

void RunRedboxFSM(Keypress_t Keycode, UWord16 SampleRate)
{
	UWord32 Temp;
	
	switch(RedboxState)
	{
		case REDBOX_WAITING_FOR_VALID_KEYPRESS:
			if(KEY_PRESS_5 == Keycode)
			{
				// compute target interval count based on sample rate
				// and selected 'coin'
				// '5' key = nickel = (1) 66 ms on/off sequence
				Temp = 66L * (UWord32)SampleRate / 1000L;
				TargetOnOffIntervalTickCount = (UWord16)Temp;
				OnOffIntervalTickCount = TargetOnOffIntervalTickCount;
				IntervalRepeatCount = 1;
				RedboxState = REDBOX_PLAYING_TONE;
			}
			else if(KEY_PRESS_1 == Keycode)
			{
				// compute target interval count based on sample rate
				// and selected 'coin'
				// '1' key = dime = (2) 66 ms on/off sequence
				Temp = 66L * (UWord32)SampleRate / 1000L;
				TargetOnOffIntervalTickCount = (UWord16)Temp;
				OnOffIntervalTickCount = TargetOnOffIntervalTickCount;
				IntervalRepeatCount = 2;
				RedboxState = REDBOX_PLAYING_TONE;
			}
			else if(KEY_PRESS_2 == Keycode)
			{
				// compute target interval count based on sample rate
				// and selected 'coin'
				// '2' key = quarter = (5) 33 ms on/off sequence
				Temp = 33L * (UWord32)SampleRate / 1000L;
				TargetOnOffIntervalTickCount = (UWord16)Temp;
				OnOffIntervalTickCount = TargetOnOffIntervalTickCount;
				IntervalRepeatCount = 5;
				RedboxState = REDBOX_PLAYING_TONE;
			}
			
			break;

		case REDBOX_PLAYING_TONE:
			OnOffIntervalTickCount--;
			
			if(0 == OnOffIntervalTickCount)
			{
				OnOffIntervalTickCount = TargetOnOffIntervalTickCount;
				RedboxState = REDBOX_PLAYING_PAUSE;
			}
		
			break;
			
		case REDBOX_PLAYING_PAUSE:
			OnOffIntervalTickCount--;
			
			if(0 == OnOffIntervalTickCount)
			{
				IntervalRepeatCount--;
				
				if(0 == IntervalRepeatCount)
				{
					RedboxState = REDBOX_WAITING_FOR_KEY_RELEASE;
				}
				else
				{
					OnOffIntervalTickCount = TargetOnOffIntervalTickCount;
					RedboxState = REDBOX_PLAYING_TONE;
				}
			}
		
			break;
			
		case REDBOX_WAITING_FOR_KEY_RELEASE:
			// TODO: we need to add a NO_KEYPRESS value to Keypress_t enumerated type
			// For now, we will cheat and use 'invalid value', as this is what will
			// be input if no key is pressed
			if(KEY_PRESS_INVALID == Keycode)
				RedboxState = REDBOX_WAITING_FOR_VALID_KEYPRESS;
				
			break;
		
		default:
			RedboxState = REDBOX_WAITING_FOR_VALID_KEYPRESS;
			break;
	}

	if(REDBOX_PLAYING_TONE == RedboxState)
		PlayTone(KEY_PRESS_1, KEYTONE_MODE_REDBOX, SampleRate);
	else
		PlayTone(KEY_PRESS_INVALID, KEYTONE_MODE_REDBOX, SampleRate);

}

#endif	//_SIMPLE_REDBOX_TEST



/* END Redbox */
